/**
 * Copyright (C) 2008  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.console.webui.commons;

import java.util.HashSet;

import org.exoplatform.web.application.Parameter;
import org.exoplatform.webui.core.UIComponent;

/**
 * NovaLink purpose is to construct the link with a row of a grid
 * 
 * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
 * @author Elwalid Brahim <elwalid.brahim@bull.net>
 */
public class NovaLink {

  /**
   * The header title of the nova link
   */
  private String header;

  /**
   * The event essociated to the nova link
   */
  private String event;

  /**
   * The parameters array for the event of the nova link
   */
  private String[] params;

  /**
   * The ui component on which the event of the nova link is sent
   */
  private UIComponent component;

  /** Constructors **/

  public NovaLink(String header, UIComponent component, String event, String[] params) {
    this.configure(header, component, event, params);
  }

  public NovaLink(String header, UIComponent component, String event) {
    this.configure(header, component, event, null);
  }

  /**
   * Configure a Nova link
   * 
   * @param the header of the action
   * @param the ui component who receive the event
   * @param the event asssociated to the action
   * @param the parameters of the event
   * @author Elwalid Brahim <elwalid.brahim@bull.net>
   * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
   */
  private void configure(String header, UIComponent component, String event, String[] params) {
    this.header = header;
    this.component = component;
    this.event = event;
    this.params = params;
  }

  /**
   * Generate an url that send an event to the receiver known by the nova link
   * 
   * @param the data associated to the event
   * @return the generated url of the event
   * @author Elwalid Brahim <elwalid.brahim@bull.net>
   * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
   */
  public String link(NovaData data) throws Exception {
    /* prepare the params */
    if (this.params == null) {
      return component.event(this.event, data.get(this.header));
    }
    HashSet<Parameter> parameters = new HashSet<Parameter>();
    for (int i = 0; i < this.params.length; i++) {
      parameters.add(new Parameter(this.params[i], data.get(this.params[i])));
    }
    return component.event(this.event, data.get(this.header), parameters.toArray(new Parameter[1]));
  }
}
