/**
 * Copyright (C) 2008  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA. 
 **/
package org.ow2.orchestra.console.webui.commons;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;

import org.exoplatform.commons.utils.ObjectPageList;
import org.exoplatform.webui.Util;
import org.exoplatform.webui.application.ConfigurationManager;
import org.exoplatform.webui.application.WebuiApplication;
import org.exoplatform.webui.application.WebuiRequestContext;
import org.exoplatform.webui.config.Component;
import org.exoplatform.webui.config.annotation.ComponentConfig;
import org.exoplatform.webui.config.annotation.ComponentConfigs;
import org.exoplatform.webui.core.UIContainer;
import org.exoplatform.webui.core.lifecycle.UIFormLifecycle;
import org.exoplatform.webui.core.model.SelectItemOption;
import org.exoplatform.webui.form.UIForm;
import org.exoplatform.webui.form.UIFormInputSet;
import org.exoplatform.webui.form.UIFormSelectBox;
import org.exoplatform.webui.form.UIFormStringInput;

/**
 * Nova Grid is a class that represent a grid of nova data
 * 
 * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
 * @author Elwalid BRAHIM <elwalid.brahim@bull.net>
 */
@ComponentConfigs({
    @ComponentConfig(id = "form", template = "system:/groovy/webui/form/UISearchForm.gtmpl", lifecycle = UIFormLifecycle.class),
    @ComponentConfig(id = "detail", template = "system:/groovy/novabpm/commons/webui/UINovaDetail.gtmpl"),
    @ComponentConfig(template = "system:/groovy/novabpm/commons/webui/UINovaGrid.gtmpl")
})
public class UINovaGrid extends UIContainer {

  /**
   * The Form
   */
  private UIForm gridForm;

  /**
   * The page iterator
   */
  private UINovaPageIterator uiIterator;

  /**
   * The headers to print
   */
  private ArrayList<String> header;

  /**
   * The headers on which we can SEARCH
   */
  private String[] search;

  /**
   * The headers on which we can order
   */
  private String[] order;

  /**
   * The headers on which we can filter
   */
  private String[] filter;

  /**
   * The headers that should be display as icon
   */
  private ArrayList<String> icons;
  /**
   * The column where print a link
   */
  private HashMap<String, NovaLink> link;

  /**
   * The action accepted in the grid
   */
  private ArrayList<String> action;

  /**
   * Determine if the action column is shown
   */
  private boolean showAction = true;

  /**
   * Determine if the search is actived
   */
  private boolean showSearch = false;

  /* ===================== */

  /**
   * data of the grid
   */
  @SuppressWarnings("unused")
  private NovaData[] data;

  /**
   * number of item by page
   */
  private int frame = 10;

  /**
   * Constructor
   * 
   * @throws Exception
   */
  public UINovaGrid() throws Exception {
    super();
    // init some members
    action = new ArrayList<String>();
    header = new ArrayList<String>();
    icons = new ArrayList<String>();
    // Creation of the form with a local configuration
    WebuiRequestContext context = WebuiRequestContext.getCurrentInstance();
    WebuiApplication app = (WebuiApplication) context.getApplication();
    ConfigurationManager configManager = app.getConfigurationManager();
    Component config = configManager.getComponentConfig(UINovaGrid.class, "form");
    this.gridForm = Util.createObject(UIForm.class, config.getInitParams());
    this.gridForm.setComponentConfig(null, config);
    config.getUIComponentLifecycle().init(this.gridForm, context);
    this.gridForm.setParent(this);

    // Add the page iterator
    uiIterator = addChild(UINovaPageIterator.class, null, null);
    // BUG FIX: the id was set to the class name...?..
    uiIterator.setId(null);
    // uiIterator_.setParent(this);
  }

  /**
   * Get the page iterator
   * 
   * @return the page iterator
   */
  public UINovaPageIterator getUIPageIterator() {
    return uiIterator;
  }

  /**
   * Return true if the actions column is visible
   * 
   * @return boolean representing the visibility of the action column
   */
  public boolean isShowAction() {
    return showAction;
  }

  /**
   * Set the visibility of the actions column
   * 
   * @param show action
   */
  public void setShowAction(boolean showAction) {
    this.showAction = showAction;
  }

  /**
   * Configure the nova grid
   * 
   * @param headers
   * @param searches
   * @param filters
   * @param orders
   * @param links
   */
  public void configure(String[] headers, String[] searches, String[] filters, String[] orders, HashMap<String, NovaLink> links) {
    setHeader(headers);
    setSearch(searches);
    setFilter(filters);
    setOrder(orders);
    setLink(links);
  }

  /**
   * Set the data list of the nova grid
   * 
   * @param data
   */
  @SuppressWarnings("unchecked")
  public void setData(Collection data) {
    // System.out.println("DATA SIZE : " + data.size());
    uiIterator.setPageList(new ObjectPageList(new ArrayList(data), frame));
  }

  /**
   * Get the data list for the nova grid
   * 
   * @return array list of nova data
   * @throws Exception
   */
  @SuppressWarnings("unchecked")
  public NovaData[] getData() throws Exception {
    return (NovaData[]) uiIterator.getCurrentPageData().toArray(new NovaData[1]);
  }

  /**
   * Get the headers list of the nova grid
   * 
   * @return array of headers
   */
  public String[] getHeader() {
    if (header.size() == 0) {
      return null;
    }
    return header.toArray(new String[1]);
  }

  /**
   * Reset the headers of the nova grid into an empty list
   */
  public void resetHeader() {
    this.header = new ArrayList<String>();
  }

  /**
   * Set the headers of the nova grid
   * 
   * @param headers
   */
  public void setHeader(String[] headers) {
    for (int i = 0; i < headers.length; i++) {
      this.header.add(headers[i]);
    }

  }

  /**
   * Add the given header to the headers of the nova grid
   * 
   * @param header to add
   */
  public void addHeader(String aHeader) {
    this.header.add(aHeader);
  }

  /**
   * Remove a header from the headers list of the nova grid
   * 
   * @param aHeader
   */
  public void removeHeader(String aHeader) {
    this.header.remove(aHeader);
  }

  /**
   * Set the iconifiable header list
   * 
   * @param array of headers
   */
  public void setIconifiable(String[] headers) {
    for (int i = 0; i < headers.length; i++) {
      this.icons.add(headers[i]);
    }
  }

  /**
   * Check if given header is iconifiable
   * 
   * @param header
   * @return boolean
   */
  public boolean isIconifiable(String header) {
    return this.icons.contains(header);
  }

  /**
   * Get the array of headers on which we can search
   * 
   * @return array of headers
   */
  public String[] getSearch() {
    if (search == null) {
      return null;
    }
    return search.clone();
  }

  /**
   * Set the list of headers on which we can do search operations.
   * 
   * @param array of headers
   */
  public void setSearch(String[] search) {
    if (search != null) {
      this.showSearch(true);
      this.search = search.clone();
  
      List<SelectItemOption<String>> options = new ArrayList<SelectItemOption<String>>();
      options.add(new SelectItemOption<String>("All", "All"));
      for (int i = 0; i < search.length; i++) {
        options.add(new SelectItemOption<String>(search[i], search[i]));
      }
      UIFormInputSet uiQuickSearchSet = new UIFormInputSet("QuickSearchSet");
      uiQuickSearchSet.addUIFormInput(new UIFormStringInput("searchTerm", null, null));
      uiQuickSearchSet.addUIFormInput(new UIFormSelectBox("searchOption", null, null).setOptions(options));
      gridForm.addChild(uiQuickSearchSet);
    }
  }

  /**
   * Get the set of search input fields for the nova grid
   * 
   * @return object representing a set of input fields
   */
  public UIFormInputSet getSearchInputSet() {
    return (UIFormInputSet) this.gridForm.getChildById("QuickSearchSet");
  }

  /**
   * Get the list of headers that we can order
   * 
   * @return array of headers
   */
  public String[] getOrder() {
    if (order == null) { 
      return null;
    }
    return order.clone();
  }

  /**
   * Set the list of headers that we can order
   * 
   * @param array of headers
   */
  public void setOrder(String[] order) {
    if (order != null) {
      this.order = order.clone();
    }
  }

  /**
   * Return true if the given header is orderable
   * 
   * @param header
   * @return boolean
   */
  public boolean isOrderable(String header) {
    if (this.order == null) {
      return false;
    }
    for (int i = 0; i < this.order.length; i++) {
      if (header.equals(this.order[i])) {
        return true;
      }
    }
    return false;
  }

  /**
   * Get the array of all the headers on what we can do filter operations
   * 
   * @return array of headers
   */
  public String[] getFilter() {
    if (filter == null) {
      return null;
    }
    return filter.clone();
  }

  /**
   * Set the array of all the headers on what we can do filter operations
   * 
   * @param array of headers
   */
  public void setFilter(String[] filter) {
    if (filter != null) {
      this.filter = filter.clone();
    }
  }

  /**
   * Get the nova links (if exists) for each headers of the nova grid
   * 
   * @return map headers <-> nova links
   */
  public HashMap<String, NovaLink> getLink() {
    if (link == null) {
      return new HashMap<String, NovaLink>();
    }
    return link;
  }

  /**
   * Set the links of the nova grid.
   * 
   * @param the map of headers <-> nova links
   */
  public void setLink(HashMap<String, NovaLink> link) {
    this.link = link;
  }

  /**
   * Return true if the search form, of the nova grid, is visible
   * 
   * @return boolean
   */
  public boolean isShowSearch() {
    return showSearch;
  }

  /**
   * Set the visibility of the search form
   * 
   * @param boolean
   */
  public void showSearch(boolean showSearch) {
    this.showSearch = showSearch;
  }

  /**
   * Check if the search form is visible or if there is at least one header on
   * which we can filter.
   * 
   * @return boolean
   */
  public boolean isShowForm() {
    if (showSearch || filter != null) {
      return true;
    }
    return false;
  }

  /**
   * Get the form
   * 
   * @return ui form
   */
  public UIForm getForm() {
    return gridForm;
  }

  /**
   * Set the list of visible actions
   * 
   * @param array of actions
   */
  public void setVisibleAction(String[] actions) {
    for (int i = 0; i < actions.length; i++) {
      action.add(actions[i]);
    }
  }

  /**
   * Get the list of the visible actions
   * 
   * @return visible actions
   */
  public Collection<String> getVisibleAction() {
    return action;
  }
}
