/**
 * Copyright (C) 2008  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.console.webui.novaview;

import java.util.Iterator;
import java.util.List;

import org.exoplatform.webui.application.WebuiRequestContext;
import org.exoplatform.webui.core.UIComponent;
import org.exoplatform.webui.core.UIContainer;

/**
 * UINovaViewContainer is a container of nova views
 * 
 * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
 * @author Elwalid BRAHIM <elwalid.brahim@bull.net>
 */
public class UINovaViewContainer extends UIContainer {

  /**
   * Define if the UINovaViewManagement can switch the child view or not
   */
  private boolean switchView = true;

  /**
   * The factory of views
   */
  private UINovaViewFactory factory;

  /**
   * The display name of the nova view container
   */
  private String displayName;

  /**
   * The type of the view
   */
  private String type = UINovaView.TYPE_FORM;

  /**
   * Get the views factory
   * 
   * @return nova view factory
   */
  public UINovaViewFactory getFactory() {
    return factory;
  }

  /**
   * Set the factory of views
   * 
   * @param factory
   */
  public void setFactory(UINovaViewFactory factory) {
    this.factory = factory;
  }

  /**
   * Return true if the nova view container can switch between its contained
   * views
   * 
   * @return boolean
   */
  public boolean isSwitchView() {
    return switchView;
  }

  /**
   * Set if the nova view container can switch between its contained views
   * 
   * @param boolean
   */
  public void setSwitchView(boolean switchView) {
    this.switchView = switchView;
  }

  /**
   * Switch between the contained views. It allow the nova view container to
   * render the view of the given id.
   * 
   * @param view identificator
   */
  public void switchView(String viewId) {
    if (isSwitchView()) {
      List<UIComponent> list = this.getChildren();
      Iterator<UIComponent> ite = list.iterator();
      while (ite.hasNext()) {
        UIComponent component = ite.next();
        if (component.getId().equals(viewId)) {
          //System.out.println("Container: "+getId()+" rend visible: "+component
          // .getId());
          component.setRendered(true);
        } else {
          component.setRendered(false);
        }
      }
    }
  }

  /**
   * This method allow to render the contained views that can be rendred
   * 
   * @param request context
   * @throws Exception
   */
  public void processRenderView(WebuiRequestContext context) throws Exception {
    List<UIComponent> list = getChildren();
    Iterator<UIComponent> ite = list.iterator();
    while (ite.hasNext()) {
      UIComponent component = ite.next();
      if (component.isRendered()) {
        component.processRender(context);
      }
    }
  }

  /**
   * The update method allow to propagate the update request to all the
   * contained views.
   * 
   * @throws Exception
   */
  public void update() throws Exception {
    List<UIComponent> children = getChildren();
    Iterator<UIComponent> ite = children.iterator();
    while (ite.hasNext()) {
      UIComponent uic = ite.next();
      if (UINovaView.class.isInstance(uic)) {
        UINovaView.class.cast(uic).update();
      } else if (UINovaViewContainer.class.isInstance(uic)) {
        UINovaViewContainer.class.cast(uic).update();
      }
    }
  }

  /**
   * Set the display name for the nova view container
   * 
   * @param display name
   */
  public void setDisplayName(String displayName) {
    this.displayName = displayName;
  }

  /**
   * Get the display name of the nova view container
   * 
   * @return display name
   */
  public String getDisplayName() {
    return this.displayName;
  }

  /**
   * get the type of the view
   * 
   * @return the type
   */
  public String getType() {
    return this.type;
  }

  /**
   * set the type of the view
   * 
   * @param type the type
   */
  public void setType(String type) {
    this.type = type;
  }

}
