package org.ow2.orchestra.console.webui.validator;

import org.exoplatform.web.application.ApplicationMessage;
import org.exoplatform.webui.core.UIComponent;
import org.exoplatform.webui.exception.MessageException;
import org.exoplatform.webui.form.UIForm;
import org.exoplatform.webui.form.UIFormInput;
import org.exoplatform.webui.form.validator.Validator;

/**
 * Created by The eXo Platform SAS
 * Author : Pham Dinh Tan
 *          pdtanit@gmail.com
 * Apr 16, 2008  
 */
/**
 * Modify by Rodrigue Le Gall
 */
public class NumberInRangeValidator implements Validator {

  /**
   * min number of range
   */
  private Long minItern = 0L;

  /**
   * max number of range
   */
  private Long maxItern = 0L;

  public NumberInRangeValidator(Long min, Long max) {
    this.maxItern = max;
    this.minItern = min;
  }

  @SuppressWarnings("unchecked")
  public void validate(UIFormInput uiInput) throws Exception {
    if (uiInput.getValue() == null || ((String) uiInput.getValue()).trim().length() == 0) {
      return;
    }
    // modified by Pham Dinh Tan
    UIComponent uiComponent = (UIComponent) uiInput;
    UIForm uiForm = uiComponent.getAncestorOfType(UIForm.class);
    String label;
    try {
      label = uiForm.getLabel(uiInput.getName());
    } catch (Exception e) {
      label = uiInput.getName();
    }
    label = label.trim();
    if (label.charAt(label.length() - 1) == ':') {
      label = label.substring(0, label.length() - 1);
    }
    String s = (String) uiInput.getValue();
    boolean error = false;
    for (int i = 0; i < s.length(); i++) {
      char c = s.charAt(i);
      if (Character.isDigit(c) || (s.charAt(0) == '-' && i == 0)) {
        error = true;
        continue;
      }
      error = false;
      Object[] args = {label, uiInput.getBindingField()};
      throw new MessageException(new ApplicationMessage("NumberFormatValidator.msg.Invalid-number", 
                                                        args, 
                                                        ApplicationMessage.WARNING));
    }
    if (error) {
      Long num = Long.decode(s);
      if (minItern > num || maxItern < num) {
        Object[] args = {label, minItern.toString(), maxItern.toString()};
        throw new MessageException(new ApplicationMessage("NumberInRangeValidator.msg.Invalid-number", 
                                                          args, 
                                                          ApplicationMessage.WARNING));
      }
    }
  }
}
